#We want to test whether the distribution in east coast states differ from other states.

####Read in data and cleaning####
real_d=da34877.0002
colnames(real_d)
state=real_d$STATE
dim(real_d)
real_d$CHGENP
summary(real_d$HHTYPEP)

#We focus on studying the household type of the East coast states: Massachusetts, New York and New Jersey
ind_mass=which(state=='(10) Massachusetts')
ind_NY=which(state=='(05) New York')
ind_NJersey=which(state=='(11) New Jersey')

house=real_d$HHTYPEP
house_mass=house[ind_mass]
house_NY=house[ind_NY]
house_NJersey=house[ind_NJersey]
house_rem=house[-c(ind_mass, ind_NY, ind_NJersey)]
length(house_mass)#386
length(house_NY)#167
length(house_NJersey)#326
length(house_rem)#2103
house_mass=house_mass[is.na(house_mass)==F] #Remove all the nan
house_NY=house_NY[is.na(house_NY)==F]
house_NJersey=house_NJersey[is.na(house_NJersey)==F]
house_rem=house_rem[is.na(house_rem)==F]
length(house_mass)#372
length(house_NY)#162
length(house_NJersey)#313
length(house_rem)#2059


##Change the data into a one-way frequency table
levels=unique(house_mass)
dat_m=summary(house_mass)
dat_ny=summary(house_NY)
dat_nj=summary(house_NJersey)
dat_rem=summary(house_rem)
n_m=sum(dat_m)
n_ny=sum(dat_ny)
n_nj=sum(dat_nj)
n_rem=sum(dat_rem)

# prop_negative=function(d){
#   count=0
#   for (i in 1:nrow(d)){
#     if (any(d[i, ]<0)){
#       count=count+1
#     }
#   }
#   return(count/nrow(d))
# }

#H0 hypothesized values using the remaining states proportions
H0=dat_rem/n_rem
#0.196, 0.603, 0.069, 0.122, 0.010

##Private data results for New York State for illustration with epsilon = 0.25, 0.5, 0.75
##We compare the performance (using L1 loss) among different methods
#Lap and TLap
eps=0.25
n=n_ny
set.seed(1)
lap_dat25=cbind(generate_laplace_nom(eps, x=dat_ny[1], size=500, n),
                generate_laplace_nom(eps, x=dat_ny[2], size=500, n),
                generate_laplace_nom(eps, x=dat_ny[3], size=500, n),
                generate_laplace_nom(eps, x=dat_ny[4], size=500, n),
                generate_laplace_nom(eps, x=dat_ny[5], size=500, n))
mu_lap25=apply(lap_dat25, 2, mean)
sd_lap25=round(mean(apply(lap_dat25, 2, sd)), 2)
prop_neg_lap25=prop_negative(lap_dat25)

lap_dat25_trunc=cbind(generate_laplace_nom_trunc(eps, x=dat_ny[1], size=500, n),
                      generate_laplace_nom_trunc(eps, x=dat_ny[2], size=500, n),
                      generate_laplace_nom_trunc(eps, x=dat_ny[3], size=500, n),
                      generate_laplace_nom_trunc(eps, x=dat_ny[4], size=500, n),
                      generate_laplace_nom_trunc(eps, x=dat_ny[5], size=500, n))
mu_lap25_trunc=apply(lap_dat25_trunc, 2, mean)
sd_lap25_trunc=round(mean(apply(lap_dat25_trunc, 2, sd)), 2)
prop_neg_lap25_trunc=prop_negative(lap_dat25_trunc)



#Opt
set.seed(1)
p=rep(1/(1+n_ny), 1+n_ny)
eps=0.25
size=500
opt_dat25=cbind(generate_optimal_data(n_ny, eps, p, size, obs=dat_ny[1]),
                generate_optimal_data(n_ny, eps, p, size, obs=dat_ny[2]),
                generate_optimal_data(n_ny, eps, p, size, obs=dat_ny[3]),
                generate_optimal_data(n_ny, eps, p, size, obs=dat_ny[4]),
                generate_optimal_data(n_ny, eps, p, size, obs=dat_ny[5]))
mu_opt25=apply(opt_dat25, 2, mean)
sd_opt25=round(mean(apply(opt_dat25, 2, sd)), 2)
prop_neg_opt25=prop_negative(opt_dat25)

#GDP and TGDP
set.seed(1)
eps=0.25
delta=1/n_ny
GDP_dat25=cbind(generate_GDP(eps, delta, dat_ny[1], size, n),
                generate_GDP(eps, delta, dat_ny[2], size, n),
                generate_GDP(eps, delta, dat_ny[3], size, n),
                generate_GDP(eps, delta, dat_ny[4], size, n),
                generate_GDP(eps, delta, dat_ny[5], size, n))
mu_GDP25=apply(GDP_dat25, 2, mean)
sd_GDP25=round(mean(apply(GDP_dat25, 2, sd)), 2)
prop_neg_GDP25=prop_negative(GDP_dat25)

set.seed(1)
GDP_dat25_trunc=cbind(generate_GDP_trunc(eps, delta, dat_ny[1], size, n),
                      generate_GDP_trunc(eps, delta, dat_ny[2], size, n),
                      generate_GDP_trunc(eps, delta, dat_ny[3], size, n),
                      generate_GDP_trunc(eps, delta, dat_ny[4], size, n),
                      generate_GDP_trunc(eps, delta, dat_ny[5], size, n))
mu_GDP25_trunc=apply(GDP_dat25_trunc, 2, mean)
sd_GDP25_trunc=round(mean(apply(GDP_dat25_trunc, 2, sd)), 2)
prop_neg_GDP25_trunc=prop_negative(GDP_dat25_trunc)


#L1 loss
opt_l1_l25=compute_L1_loss(opt_dat25, dat_ny) 
lap_l1_l25=compute_L1_loss(lap_dat25, dat_ny)
GDP_l1_l25=compute_L1_loss(GDP_dat25, dat_ny)
lap_l1_l25_trunc=compute_L1_loss(lap_dat25_trunc, dat_ny)
GDP_l1_l25_trunc=compute_L1_loss(GDP_dat25_trunc, dat_ny)
loss_lap=mean(apply(lap_l1_l25, 2, mean))
loss_opt=mean(apply(opt_l1_l25, 2, mean))
loss_GDP=mean(apply(GDP_l1_l25, 2, mean))
loss_lap_trunc=mean(apply(lap_l1_l25_trunc, 2, mean))
loss_GDP_trunc=mean(apply(GDP_l1_l25_trunc, 2, mean))

table_ny25=rbind(round(c(mu_opt25, sd_opt25,loss_opt,prop_neg_opt25), 2),
                 round(c(mu_lap25, sd_lap25,loss_lap, prop_neg_lap25), 2),
                 round(c(mu_lap25_trunc, sd_lap25_trunc,loss_lap_trunc, prop_neg_lap25_trunc), 2),
                 round(c(mu_GDP25, sd_GDP25,loss_GDP,prop_neg_GDP25), 2),
                 round(c(mu_GDP25_trunc, sd_GDP25_trunc,loss_GDP_trunc,prop_neg_GDP25_trunc), 2))

table_ny25


### When eps=0.5 ###
#Lap and TLap
set.seed(1)
eps=0.5
n=n_ny
lap_dat5=cbind(generate_laplace_nom(eps, x=dat_ny[1], size=500, n),
               generate_laplace_nom(eps, x=dat_ny[2], size=500, n),
               generate_laplace_nom(eps, x=dat_ny[3], size=500, n),
               generate_laplace_nom(eps, x=dat_ny[4], size=500, n),
               generate_laplace_nom(eps, x=dat_ny[5], size=500, n))
mu_lap=apply(lap_dat5, 2, mean)
sd_lap=round(mean(apply(lap_dat5, 2, sd)), 2)
prop_neg_lap5=prop_negative(lap_dat5)

set.seed(1)
eps=0.5
n=n_ny
lap_dat5_trunc=cbind(generate_laplace_nom_trunc(eps, x=dat_ny[1], size=500, n),
                     generate_laplace_nom_trunc(eps, x=dat_ny[2], size=500, n),
                     generate_laplace_nom_trunc(eps, x=dat_ny[3], size=500, n),
                     generate_laplace_nom_trunc(eps, x=dat_ny[4], size=500, n),
                     generate_laplace_nom_trunc(eps, x=dat_ny[5], size=500, n))
mu_lap_trunc=apply(lap_dat5_trunc, 2, mean)
sd_lap_trunc=round(mean(apply(lap_dat5_trunc, 2, sd)), 2)
prop_neg_lap5_trunc=prop_negative(lap_dat5_trunc)

#Opt
set.seed(1)
p=rep(1/(1+n_ny), 1+n_ny)
eps=0.5
size=500
opt_dat5=cbind(generate_optimal_data(n_ny, eps, p, size, obs=dat_ny[1]),
               generate_optimal_data(n_ny, eps, p, size, obs=dat_ny[2]),
               generate_optimal_data(n_ny, eps, p, size, obs=dat_ny[3]),
               generate_optimal_data(n_ny, eps, p, size, obs=dat_ny[4]),
               generate_optimal_data(n_ny, eps, p, size, obs=dat_ny[5]))
mu_opt=apply(opt_dat5, 2, mean)
sd_opt=round(mean(apply(opt_dat5, 2, sd)), 2)
prop_neg_opt5=prop_negative(opt_dat5)

#GDP and TGDP
set.seed(1)
eps=0.5
delta=1/n_ny
GDP_dat5=cbind(generate_GDP(eps, delta, dat_ny[1], size, n),
               generate_GDP(eps, delta, dat_ny[2], size, n),
               generate_GDP(eps, delta, dat_ny[3], size, n),
               generate_GDP(eps, delta, dat_ny[4], size, n),
               generate_GDP(eps, delta, dat_ny[5], size, n))
mu_GDP=apply(GDP_dat5, 2, mean)
sd_GDP=round(mean(apply(GDP_dat5, 2, sd)), 2)
prop_neg_GDP5=prop_negative(GDP_dat5)

set.seed(1)
eps=0.5
delta=1/n_ny
GDP_dat5_trunc=cbind(generate_GDP_trunc(eps, delta, dat_ny[1], size, n),
                     generate_GDP_trunc(eps, delta, dat_ny[2], size, n),
                     generate_GDP_trunc(eps, delta, dat_ny[3], size, n),
                     generate_GDP_trunc(eps, delta, dat_ny[4], size, n),
                     generate_GDP_trunc(eps, delta, dat_ny[5], size, n))
mu_GDP_trunc=apply(GDP_dat5_trunc, 2, mean)
sd_GDP_trunc=round(mean(apply(GDP_dat5_trunc, 2, sd)), 2)
prop_neg_GDP5_trunc=prop_negative(GDP_dat5_trunc)


#L1 loss
opt_l1_l5=compute_L1_loss(opt_dat5, dat_ny) 
lap_l1_l5=compute_L1_loss(lap_dat5, dat_ny)
GDP_l1_l5=compute_L1_loss(GDP_dat5, dat_ny)
lap_l1_l5_trunc=compute_L1_loss(lap_dat5_trunc, dat_ny)
GDP_l1_l5_trunc=compute_L1_loss(GDP_dat5_trunc, dat_ny)
loss_lap=mean(apply(lap_l1_l5, 2, mean))
loss_opt=mean(apply(opt_l1_l5, 2, mean))
loss_GDP=mean(apply(GDP_l1_l5, 2, mean))
loss_lap_trunc=mean(apply(lap_l1_l5_trunc, 2, mean))
loss_GDP_trunc=mean(apply(GDP_l1_l5_trunc, 2, mean))

table_ny5=rbind(round(c(mu_opt, sd_opt,loss_opt,prop_neg_opt5), 2),
                round(c(mu_lap, sd_lap,loss_lap,prop_neg_lap5), 2),
                round(c(mu_lap_trunc, sd_lap_trunc,loss_lap_trunc,prop_neg_lap5_trunc), 2),
                round(c(mu_GDP, sd_GDP,loss_GDP,prop_neg_GDP5), 2),
                round(c(mu_GDP_trunc, sd_GDP_trunc,loss_GDP_trunc,prop_neg_GDP5_trunc), 2))

table_ny5


##When eps=0.75
#Lap and TLap
eps=0.75
set.seed(1)
n=n_ny
lap_dat75_trunc=cbind(generate_laplace_nom_trunc(eps, x=dat_ny[1], size=500, n),
                     generate_laplace_nom_trunc(eps, x=dat_ny[2], size=500, n),
                     generate_laplace_nom_trunc(eps, x=dat_ny[3], size=500, n),
                     generate_laplace_nom_trunc(eps, x=dat_ny[4], size=500, n),
                     generate_laplace_nom_trunc(eps, x=dat_ny[5], size=500, n))
mu_lap_trunc=apply(lap_dat75_trunc, 2, mean)
sd_lap_trunc=round(mean(apply(lap_dat75_trunc, 2, sd)), 2)
prop_neg_lap75_trunc=prop_negative(lap_dat75_trunc)


eps=0.75
set.seed(1)
n=n_ny
lap_dat75=cbind(generate_laplace_nom(eps, x=dat_ny[1], size=500, n),
               generate_laplace_nom(eps, x=dat_ny[2], size=500, n),
               generate_laplace_nom(eps, x=dat_ny[3], size=500, n),
               generate_laplace_nom(eps, x=dat_ny[4], size=500, n),
               generate_laplace_nom(eps, x=dat_ny[5], size=500, n))
mu_lap=apply(lap_dat75, 2, mean)
sd_lap=round(mean(apply(lap_dat75, 2, sd)), 2)
prop_neg_lap75=prop_negative(lap_dat75)

#Opt
set.seed(1)
p=rep(1/(1+n_ny), 1+n_ny)
eps=0.75
size=500
opt_dat75=cbind(generate_optimal_data(n_ny, eps, p, size, obs=dat_ny[1]),
               generate_optimal_data(n_ny, eps, p, size, obs=dat_ny[2]),
               generate_optimal_data(n_ny, eps, p, size, obs=dat_ny[3]),
               generate_optimal_data(n_ny, eps, p, size, obs=dat_ny[4]),
               generate_optimal_data(n_ny, eps, p, size, obs=dat_ny[5]))
mu_opt=apply(opt_dat75, 2, mean)
sd_opt=round(mean(apply(opt_dat75, 2, sd)), 2)
prop_neg_opt75=prop_negative(opt_dat75)

#GDP and TGDP
set.seed(1)
eps=0.75
delta=1/n_ny
GDP_dat75=cbind(generate_GDP(eps, delta, dat_ny[1], size, n),
               generate_GDP(eps, delta, dat_ny[2], size, n),
               generate_GDP(eps, delta, dat_ny[3], size, n),
               generate_GDP(eps, delta, dat_ny[4], size, n),
               generate_GDP(eps, delta, dat_ny[5], size, n))
mu_GDP=apply(GDP_dat75, 2, mean)
sd_GDP=round(mean(apply(GDP_dat75, 2, sd)), 2)
prop_neg_GDP75=prop_negative(GDP_dat75)


set.seed(1)
eps=0.75
delta=1/n_ny
GDP_dat75_trunc=cbind(generate_GDP_trunc(eps, delta, dat_ny[1], size, n),
                     generate_GDP_trunc(eps, delta, dat_ny[2], size, n),
                     generate_GDP_trunc(eps, delta, dat_ny[3], size, n),
                     generate_GDP_trunc(eps, delta, dat_ny[4], size, n),
                     generate_GDP_trunc(eps, delta, dat_ny[5], size, n))
mu_GDP_trunc=apply(GDP_dat75_trunc, 2, mean)
sd_GDP_trunc=round(mean(apply(GDP_dat75_trunc, 2, sd)), 2)
prop_neg_GDP75_trunc=prop_negative(GDP_dat75_trunc)


#L1 loss
opt_l1_l75=compute_L1_loss(opt_dat75, dat_ny) 
lap_l1_l75=compute_L1_loss(lap_dat75, dat_ny)
GDP_l1_l75=compute_L1_loss(GDP_dat75, dat_ny)
lap_l1_l75_trunc=compute_L1_loss(lap_dat75_trunc, dat_ny)
GDP_l1_l75_trunc=compute_L1_loss(GDP_dat75_trunc, dat_ny)
loss_lap=mean(apply(lap_l1_l75, 2, mean))
loss_opt=mean(apply(opt_l1_l75, 2, mean))
loss_GDP=mean(apply(GDP_l1_l75, 2, mean))
loss_lap_trunc=mean(apply(lap_l1_l75_trunc, 2, mean))
loss_GDP_trunc=mean(apply(GDP_l1_l75_trunc, 2, mean))

table_ny75=rbind(round(c(mu_opt, sd_opt,loss_opt,prop_neg_opt75), 2),
                 round(c(mu_lap, sd_lap,loss_lap,prop_neg_lap75), 2),
                 round(c(mu_lap_trunc, sd_lap_trunc,loss_lap_trunc,prop_neg_lap75_trunc), 2),
                 round(c(mu_GDP, sd_GDP,loss_GDP,prop_neg_GDP75), 2),
                 round(c(mu_GDP_trunc, sd_GDP_trunc,loss_GDP_trunc,prop_neg_GDP75_trunc), 2))

table_ny75




###We combine the test statistic for household type variable to jointly test goodness of fit of all the three states in the east coast###

#Empirical type I errors
#Lap
set.seed(1)
res_lap_multi_typeI_n=c(1,1,1)
for (i in 1:100){
  typeI_c=c(typeI_combine_lap(eps=0.25, n1=n_rem, n2=n_rem, n3=n_rem, p01=H0,
                              p02=H0,p03=H0, alpha=0.05, size=500),
            typeI_combine_lap(eps=0.5, n1=n_rem, n2=n_rem, n3=n_rem, p01=H0,
                              p02=H0,p03=H0, alpha=0.05, size=500),
            typeI_combine_lap(eps=0.75, n1=n_rem, n2=n_rem, n3=n_rem, p01=H0,
                              p02=H0,p03=H0, alpha=0.05, size=500))
  res_lap_multi_typeI_n=rbind(res_lap_multi_typeI_n, typeI_c)
}
res_lap_multi_typeI_n=res_lap_multi_typeI_n[2:101,]
round(apply(res_lap_multi_typeI_n, 2, mean)/100,3)



#TLap
set.seed(1)
res_lap_multi_typeI_n_trunc=c(1,1,1)
for (i in 1:100){
  typeI_c=c(typeI_combine_lap_trunc(eps=0.25, n1=n_rem, n2=n_rem, n3=n_rem, p01=H0,
                                    p02=H0,p03=H0, alpha=0.05, size=500),
            typeI_combine_lap_trunc(eps=0.5, n1=n_rem, n2=n_rem, n3=n_rem, p01=H0,
                                    p02=H0,p03=H0, alpha=0.05, size=500),
            typeI_combine_lap_trunc(eps=0.75, n1=n_rem, n2=n_rem, n3=n_rem, p01=H0,
                                    p02=H0,p03=H0, alpha=0.05, size=500))
  res_lap_multi_typeI_n_trunc=rbind(res_lap_multi_typeI_n_trunc, typeI_c)
}
res_lap_multi_typeI_n_trunc=res_lap_multi_typeI_n_trunc[2:101,]
round(apply(res_lap_multi_typeI_n_trunc, 2, mean)/100,3) 





#GDP
size=500
alpha=0.05
set.seed(1)
res_GDP_multi_typeI_n=c(1,1,1)
delta=1/n
for (i in 1:100){
  typeI_c=c(type1_combine_GDP(eps=0.25, size=500, del1=1/n_rem, del2=1/n_rem, del3=1/n_rem, n1=n_rem, n2=n_rem,n3=n_rem, p01=dat_m/n_m,p02=dat_ny/n_ny,p03=dat_nj/n_nj,
                              alpha=0.05),
            type1_combine_GDP(eps=0.5, size=500, del1=1/n_rem, del2=1/n_rem, del3=1/n_rem, n1=n_rem, n2=n_rem,n3=n_rem, p01=dat_m/n_m,p02=dat_ny/n_ny,p03=dat_nj/n_nj,
                              alpha=0.05),
            type1_combine_GDP(eps=0.75, size=500, del1=1/n_rem, del2=1/n_rem, del3=1/n_rem, n1=n_rem, n2=n_rem,n3=n_rem, p01=dat_m/n_m,p02=dat_ny/n_ny,p03=dat_nj/n_nj,
                              alpha=0.05))
  res_GDP_multi_typeI_n=rbind(res_GDP_multi_typeI_n, typeI_c)
}
res_GDP_multi_typeI_n=res_GDP_multi_typeI_n[2:101,]
round(apply(res_GDP_multi_typeI_n, 2, mean)/100,3) 

#TGDP
size=500
alpha=0.05
set.seed(1)
res_GDP_multi_typeI_n_trunc=c(1,1,1)
delta=1/n
for (i in 1:100){
  typeI_c=c(type1_combine_GDP_trunc(eps=0.25, size=500, del1=1/n_rem, del2=1/n_rem, del3=1/n_rem, n1=n_rem, n2=n_rem,n3=n_rem, p01=dat_m/n_m,p02=dat_ny/n_ny,p03=dat_nj/n_nj,
                                    alpha=0.05),
            type1_combine_GDP_trunc(eps=0.5, size=500, del1=1/n_rem, del2=1/n_rem, del3=1/n_rem, n1=n_rem, n2=n_rem,n3=n_rem, p01=dat_m/n_m,p02=dat_ny/n_ny,p03=dat_nj/n_nj,
                                    alpha=0.05),
            type1_combine_GDP_trunc(eps=0.75, size=500, del1=1/n_rem, del2=1/n_rem, del3=1/n_rem, n1=n_rem, n2=n_rem,n3=n_rem, p01=dat_m/n_m,p02=dat_ny/n_ny,p03=dat_nj/n_nj,
                                    alpha=0.05))
  res_GDP_multi_typeI_n_trunc=rbind(res_GDP_multi_typeI_n_trunc, typeI_c)
}
res_GDP_multi_typeI_n_trunc=res_GDP_multi_typeI_n_trunc[2:101,]
round(apply(res_GDP_multi_typeI_n_trunc, 2, mean)/100,3) 



#Opt mechanism
size=500
alpha=0.05
p=rep(1/(n+1), n+1)
res_opt_typeI_n100=c(1,1,1)
for (i in 1:100){
  typeI_c=c(typeI_combine_opt(eps=0.25, size=500, n1=n_rem, n2=n_rem,n3=n_rem, p01=H0,p02=H0,p03=H0,
                              alpha=0.05, p1=rep(1/(n_rem+1), n_rem+1), p2=rep(1/(n_rem+1), n_rem+1), p3=rep(1/(n_rem+1), n_rem+1)),
            typeI_combine_opt(eps=0.5, size=500, n1=n_rem, n2=n_rem,n3=n_rem, p01=H0,p02=H0,p03=H0,
                              alpha=0.05, p1=rep(1/(n_rem+1), n_rem+1), p2=rep(1/(n_rem+1), n_rem+1), p3=rep(1/(n_rem+1), n_rem+1)),
            typeI_combine_opt(eps=0.75, size=500, n1=n_rem, n2=n_rem,n3=n_rem, p01=H0,p02=H0,p03=H0,
                              alpha=0.05, p1=rep(1/(n_rem+1), n_rem+1), p2=rep(1/(n_rem+1), n_rem+1), p3=rep(1/(n_rem+1), n_rem+1))
  )
  res_opt_typeI_n100=rbind(res_opt_typeI_n100, typeI_c)
}
res_opt_typeI_n100=res_opt_typeI_n100[2:101,]
round(apply(res_opt_typeI_n100, 2, mean)/100,3) 



###We further evaluate empirical power###
##Lap
#eps=0.25
size=500
alpha=0.05
p0=H0
p_v=seq(p0[2], 0.6527198 , 0.005)
eps=0.25
set.seed(5)
p_n100_eps0.25_lap=c()
for (p in p_v){
  delta=p-p0[2]
  p1=c(p0[1],p, p0[3:5],p0, p0[1]-delta, p0[2:5])
  pow=power_combine_lap(eps, n1=n_rem, n2=n_rem, n3=n_rem, p01=H0,p02=H0,p03=H0, 
                        p11=p1[1:5], p12=p1[6:10],p13=p1[11:15], alpha=0.05, size=500)
  p_n100_eps0.25_lap=c(p_n100_eps0.25_lap, pow)
}

#eps=0.5
eps=0.5
set.seed(5)
p_n100_eps0.5_lap=c()
for (p in p_v){
  delta=p-p0[2]
  p1=c(p0[1],p, p0[3:5],p0, p0[1]-delta, p0[2:5])
  pow=power_combine_lap(eps=0.5, n1=n_rem, n2=n_rem, n3=n_rem, p01=H0,p02=H0,p03=H0, 
                        p11=p1[1:5], p12=p1[6:10],p13=p1[11:15], alpha=0.05, size=500)
  p_n100_eps0.5_lap=c(p_n100_eps0.5_lap, pow)
}

#eps=0.75
eps=0.75
set.seed(5)
p_n100_eps0.75_lap=c()
for (p in p_v){
  delta=p-p0[2]
  p1=c(p0[1],p, p0[3:5],p0, p0[1]-delta, p0[2:5])
  pow=power_combine_lap(eps=0.75, n1=n_rem, n2=n_rem, n3=n_rem, p01=H0,p02=H0,p03=H0, 
                        p11=p1[1:5], p12=p1[6:10],p13=p1[11:15], alpha=0.05, size=500)
  p_n100_eps0.75_lap=c(p_n100_eps0.75_lap, pow)
}



#TLap
#eps=0.25
size=500
alpha=0.05
p0=H0
p_v=seq(p0[2], 0.6527198 , 0.005)
eps=0.25
set.seed(5)
p_n100_eps0.25_lap_trunc=c()
for (p in p_v){
  delta=p-p0[2]
  p1=c(p0[1],p, p0[3:5],p0, p0[1]-delta, p0[2:5])
  pow=power_combine_lap_trunc(eps, n1=n_rem, n2=n_rem, n3=n_rem, p01=H0,p02=H0,p03=H0, 
                              p11=p1[1:5], p12=p1[6:10],p13=p1[11:15], alpha=0.05, size=500)
  p_n100_eps0.25_lap_trunc=c(p_n100_eps0.25_lap_trunc, pow)
}

#eps=0.5
eps=0.5
set.seed(5)
p_n100_eps0.5_lap_trunc=c()
for (p in p_v){
  delta=p-p0[2]
  p1=c(p0[1],p, p0[3:5],p0, p0[1]-delta, p0[2:5])
  pow=power_combine_lap_trunc(eps=0.5, n1=n_rem, n2=n_rem, n3=n_rem, p01=H0,p02=H0,p03=H0, 
                              p11=p1[1:5], p12=p1[6:10],p13=p1[11:15], alpha=0.05, size=500)
  p_n100_eps0.5_lap_trunc=c(p_n100_eps0.5_lap_trunc, pow)
}

#eps=0.75
eps=0.75
set.seed(5)
p_n100_eps0.75_lap_trunc=c()
for (p in p_v){
  delta=p-p0[2]
  p1=c(p0[1],p, p0[3:5],p0, p0[1]-delta, p0[2:5])
  pow=power_combine_lap_trunc(eps=0.75, n1=n_rem, n2=n_rem, n3=n_rem, p01=H0,p02=H0,p03=H0, 
                              p11=p1[1:5], p12=p1[6:10],p13=p1[11:15], alpha=0.05, size=500)
  p_n100_eps0.75_lap_trunc=c(p_n100_eps0.75_lap_trunc, pow)
}





#GDP
#eps=0.25
size=500
alpha=0.05
p0=H0
p_v=seq(p0[2], 0.6527198 , 0.005)
delta=1/n_rem
eps=0.25
set.seed(1)
p_n100_eps0.25_GDP=c()
for (p in p_v){
  d=p-p0[2]
  p1=c(p0[1],p, p0[3:5],p0, p0[1]-d, p0[2:5])
  pow=pow_combine_GDP(eps=0.25, size=500, del1=delta, del2=delta, del3=delta,n1=n_rem, n2=n_rem,n3=n_rem, p01=H0,p02=H0,p03=H0,
                      p11=p1[1:5], p12=p1[6:10],p13=p1[11:15], alpha=0.05)
  p_n100_eps0.25_GDP=c(p_n100_eps0.25_GDP, pow)
  
}

#eps=0.5
eps=0.5
set.seed(1)
p_n100_eps0.5_GDP=c()
for (p in p_v){
  d=p-p0[2]
  p1=c(p0[1],p, p0[3:5],p0, p0[1]-d, p0[2:5])
  pow=pow_combine_GDP(eps=0.5, size=500, del1=delta, del2=delta, del3=delta,n1=n_rem, n2=n_rem,n3=n_rem, p01=H0,p02=H0,p03=H0,
                      p11=p1[1:5], p12=p1[6:10],p13=p1[11:15], alpha=0.05)
  p_n100_eps0.5_GDP=c(p_n100_eps0.5_GDP, pow)
}

#eps=0.75
eps=0.75
set.seed(1)
p_n100_eps0.75_GDP=c()
for (p in p_v){
  d=p-p0[2]
  p1=c(p0[1],p, p0[3:5],p0, p0[1]-d, p0[2:5])
  pow=pow_combine_GDP(eps=0.75, size=500, del1=delta, del2=delta, del3=delta, n1=n_rem, n2=n_rem,n3=n_rem, p01=H0,p02=H0,p03=H0,
                      p11=p1[1:5], p12=p1[6:10],p13=p1[11:15], alpha=0.05)
  p_n100_eps0.75_GDP=c(p_n100_eps0.75_GDP, pow)
}


#TGDP
#eps=0.25
size=500
alpha=0.05
p0=H0
p_v=seq(p0[2], 0.6527198 , 0.005)
delta=1/n_rem
eps=0.25
set.seed(1)
p_n100_eps0.25_GDP_trunc=c()
for (p in p_v){
  d=p-p0[2]
  p1=c(p0[1],p, p0[3:5],p0, p0[1]-d, p0[2:5])
  pow=pow_combine_GDP_trunc(eps=0.25, size=500, del1=delta, del2=delta, del3=delta,n1=n_rem, n2=n_rem,n3=n_rem, p01=H0,p02=H0,p03=H0,
                            p11=p1[1:5], p12=p1[6:10],p13=p1[11:15], alpha=0.05)
  p_n100_eps0.25_GDP_trunc=c(p_n100_eps0.25_GDP_trunc, pow)
  
}

#eps=0.5
eps=0.5
set.seed(1)
p_n100_eps0.5_GDP_trunc=c()
for (p in p_v){
  d=p-p0[2]
  p1=c(p0[1],p, p0[3:5],p0, p0[1]-d, p0[2:5])
  pow=pow_combine_GDP_trunc(eps=0.5, size=500, del1=delta, del2=delta, del3=delta,n1=n_rem, n2=n_rem,n3=n_rem, p01=H0,p02=H0,p03=H0,
                            p11=p1[1:5], p12=p1[6:10],p13=p1[11:15], alpha=0.05)
  p_n100_eps0.5_GDP_trunc=c(p_n100_eps0.5_GDP_trunc, pow)
}

#eps=0.75
eps=0.75
set.seed(1)
p_n100_eps0.75_GDP_trunc=c()
for (p in p_v){
  d=p-p0[2]
  p1=c(p0[1],p, p0[3:5],p0, p0[1]-d, p0[2:5])
  pow=pow_combine_GDP_trunc(eps=0.75, size=500, del1=delta, del2=delta, del3=delta, n1=n_rem, n2=n_rem,n3=n_rem, p01=H0,p02=H0,p03=H0,
                            p11=p1[1:5], p12=p1[6:10],p13=p1[11:15], alpha=0.05)
  p_n100_eps0.75_GDP_trunc=c(p_n100_eps0.75_GDP_trunc, pow)
}




#Opt mechanism
#eps=0.25
eps=0.25
pr=rep(1/(n_rem+1), n_rem+1)
opt_mat=perturbation_matrix(n_rem, eps, pr)
set.seed(2)
p_n100_eps0.25_opt=c()
for (p in p_v){
  d=p-p0[2]
  p1=c(p0[1],p, p0[3:5],p0, p0[1]-d, p0[2:5])
  pow=pow_combine_opt(eps=0.25, size=100, n1=n_rem, n2=n_rem,n3=n_rem, p01=H0,p02=H0,p03=H0,
                      p11=p1[1:5], p12=p1[6:10],p13=p1[11:15], alpha=0.05, opt_mat, opt_mat,opt_mat)
  
  p_n100_eps0.25_opt=c(p_n100_eps0.25_opt, pow)
}


#eps=0.5,
set.seed(1)
eps=0.5
pr=rep(1/(n_rem+1), n_rem+1)
opt_mat=perturbation_matrix(n_rem, eps, pr)
p_n100_eps0.5_opt_c=c()
for (p in p_v){
  d=p-p0[2]
  p1=c(p0[1],p, p0[3:5],p0, p0[1]-d, p0[2:5])
  pow=pow_combine_opt(eps=0.5, size=100, n1=n_rem, n2=n_rem,n3=n_rem, p01=H0,p02=H0,p03=H0,
                      p11=p1[1:5], p12=p1[6:10],p13=p1[11:15], alpha=0.05,opt_mat, opt_mat,opt_mat)
  
  p_n100_eps0.5_opt_c=c(p_n100_eps0.5_opt_c, pow)
}

#eps=0.75, 
set.seed(1)
eps=0.75
pr=rep(1/(n_rem+1), n_rem+1)
opt_mat=perturbation_matrix(n_rem, eps, pr)
p_n100_eps0.75_opt_c=c()
for (p in p_v){
  d=p-p0[2]
  p1=c(p0[1],p, p0[3:5],p0, p0[1]-d, p0[2:5])
  pow=pow_combine_opt(eps=0.75, size=200, n1=n_rem, n2=n_rem,n3=n_rem, p01=H0,p02=H0,p03=H0,
                      p11=p1[1:5], p12=p1[6:10],p13=p1[11:15], alpha=0.05, opt_mat, opt_mat,opt_mat)
  
  p_n100_eps0.75_opt_c=c(p_n100_eps0.75_opt_c, pow)
}





#Plot power multiple cells
pdf("real_power_household_latest_new.pdf", width=12, height=4)
#par(mfrow=c(1,3),cex=0.7, mai=c(0.6,0.6,0.6,0.6))
par(mfrow=c(1,3),cex=0.7, mai=c(0.6,0.25,0.2,0.2))
plot(p_v, p_n100_eps0.25_opt/100, type='o', ylim=c(0,1),
     xlab='Alternative Hypothesis (p2)', ylab='Empirical power', pch='o',
     main = 'Epsilon=0.25', lty=1, col='blue')
points(p_v,  p_n100_eps0.25_lap/100, col="red", pch="*")
lines(p_v, p_n100_eps0.25_lap/100, col="red",lty=2)
points(p_v,  p_n100_eps0.25_lap_trunc/100, col="darkgoldenrod1",pch=16)
lines(p_v,  p_n100_eps0.25_lap_trunc/100, col="darkgoldenrod1", lty=5)
points(p_v,  p_n100_eps0.25_GDP/100, col="black",pch="+")
lines(p_v,  p_n100_eps0.25_GDP/100, col="black", lty=3)
points(p_v,  p_n100_eps0.25_GDP_trunc/100, col="purple",pch=5)
lines(p_v,  p_n100_eps0.25_GDP_trunc/100, col="purple", lty=4)

legend(0.603,1,legend=c("Opt","Lap","TLap","GDP","TGDP"), col=c("blue","red","darkgoldenrod1","black","purple"),
       pch=c("o","*",16, "+",5),lty=c(1,2,5, 3,4), ncol=1,cex=0.75)

plot(p_v, p_n100_eps0.5_opt_c/100, type='o', ylim=c(0,1),
     xlab='Alternative Hypothesis (p2)', ylab='Empirical power', pch='o',
     main = 'Epsilon=0.5', lty=1, col='blue')
points(p_v,  p_n100_eps0.5_lap/100, col="red", pch="*")
lines(p_v, p_n100_eps0.5_lap/100, col="red",lty=2)
points(p_v,  p_n100_eps0.5_lap_trunc/100, col="darkgoldenrod1",pch=16)
lines(p_v,  p_n100_eps0.5_lap_trunc/100, col="darkgoldenrod1", lty=5)
points(p_v,  p_n100_eps0.5_GDP/100, col="black",pch="+")
lines(p_v,  p_n100_eps0.5_GDP/100, col="black", lty=3)
points(p_v,  p_n100_eps0.5_GDP_trunc/100, col="purple",pch=5)
lines(p_v,  p_n100_eps0.5_GDP_trunc/100, col="purple", lty=4)

legend(0.603,1,legend=c("Opt","Lap","TLap","GDP","TGDP"), col=c("blue","red","darkgoldenrod1","black","purple"),
       pch=c("o","*",16, "+",5),lty=c(1,2,5, 3,4), ncol=1,cex=0.75)

plot(p_v, p_n100_eps0.75_opt_c/100, type='o', ylim=c(0,1),
     xlab='Alternative Hypothesis (p2)', ylab='Empirical power', pch='o',
     main = 'Epsilon=0.75', lty=1, col='blue')
points(p_v,  p_n100_eps0.75_lap/100, col="red", pch="*")
lines(p_v, p_n100_eps0.75_lap/100, col="red",lty=2)
points(p_v,  p_n100_eps0.75_lap_trunc/100, col="darkgoldenrod1",pch=16)
lines(p_v,  p_n100_eps0.75_lap_trunc/100, col="darkgoldenrod1", lty=5)
points(p_v,  p_n100_eps0.75_GDP/100, col="black",pch="+")
lines(p_v,  p_n100_eps0.75_GDP/100, col="black", lty=3)
points(p_v,  p_n100_eps0.75_GDP_trunc/100, col="purple",pch=5)
lines(p_v,  p_n100_eps0.75_GDP_trunc/100, col="purple", lty=4)

legend(0.603, 1, legend=c("Opt","Lap","TLap","GDP","TGDP"), col=c("blue","red","darkgoldenrod1","black","purple"),
       pch=c("o","*",16, "+",5), lty=c(1,2,5, 3,4), ncol=1, cex=0.75)

dev.off()









###Compare average p_value obtained on the private data to the true p_value###
#On true data
set.seed(1)
t_true=sum((dat_m-n_m*p0)^2/(n_m*p0))+sum((dat_ny-n_ny*p0)^2/(n_ny*p0))+sum((dat_nj-n_nj*p0)^2/(n_nj*p0))
sts=rchisq(100000, 4)+rchisq(100000, 4)+rchisq(100000, 4)
sts=sort(sts)
1-which(sts>t_true)[1]/100000#6e-05

####On private data####
#Lap
set.seed(2)
p_val=c(p_value_combine_lap(eps=0.25, n1=n_m, n2=n_ny, n3=n_nj, p01=H0,p02=H0,p03=H0, 
                            x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj, alpha=0.05, size=500),
        p_value_combine_lap(eps=0.5, n1=n_m, n2=n_ny, n3=n_nj, p01=H0,p02=H0,p03=H0, 
                            x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj, alpha=0.05, size=500),
        p_value_combine_lap(eps=0.75, n1=n_m, n2=n_ny, n3=n_nj, p01=H0,p02=H0,p03=H0, 
                            x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj, alpha=0.05, size=500))
round(p_val,3) #0.326 0.068 0.006

#TLap
set.seed(1)
p_val=c(p_value_combine_lap_trunc(eps=0.25, n1=n_m, n2=n_ny, n3=n_nj, p01=H0,p02=H0,p03=H0, 
                                  x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj, alpha=0.05, size=500),
        p_value_combine_lap_trunc(eps=0.5, n1=n_m, n2=n_ny, n3=n_nj, p01=H0,p02=H0,p03=H0, 
                                  x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj, alpha=0.05, size=500),
        p_value_combine_lap_trunc(eps=0.75, n1=n_m, n2=n_ny, n3=n_nj, p01=H0,p02=H0,p03=H0, 
                                  x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj, alpha=0.05, size=500))
round(p_val,3) #0.394 0.054 0.010


#Opt mechanism
pr1=rep(1/(n_m+1), n_m+1)
pr2=rep(1/(n_ny+1), n_ny+1)
pr3=rep(1/(n_nj+1), n_nj+1)
opt_mat125=perturbation_matrix(n_m, eps=0.25, pr1)
opt_mat15=perturbation_matrix(n_m, eps=0.5, pr1)
opt_mat175=perturbation_matrix(n_m, eps=0.75, pr1)
opt_mat225=perturbation_matrix(n_ny, eps=0.25, pr2)
opt_mat25=perturbation_matrix(n_ny, eps=0.5, pr2)
opt_mat275=perturbation_matrix(n_ny, eps=0.75, pr2)
opt_mat325=perturbation_matrix(n_nj, eps=0.25, pr3)
opt_mat35=perturbation_matrix(n_nj, eps=0.5, pr3)
opt_mat375=perturbation_matrix(n_nj, eps=0.75, pr3)
set.seed(9)
p_val25=p_val_combine_opt(eps=0.25, n1=n_m, n2=n_ny, n3=n_nj, p01=H0,p02=H0,p03=H0, 
                          x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj, alpha=0.05, size=500, 
                          opt_mat125,opt_mat225,opt_mat325)
p_val5=p_val_combine_opt(eps=0.5, n1=n_m, n2=n_ny, n3=n_nj, p01=H0,p02=H0,p03=H0, 
                          x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj, alpha=0.05, size=500, 
                          opt_mat15,opt_mat25,opt_mat35)
set.seed(4)
p_val75=p_val_combine_opt(eps=0.75, n1=n_m, n2=n_ny, n3=n_nj, p01=H0,p02=H0,p03=H0, 
                          x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj, alpha=0.05, size=500, 
                          opt_mat175,opt_mat275,opt_mat375)
p_val=c(p_val25, p_val5,p_val75)
round(p_val,3) #0.372 0.042 0.006


#GDP
set.seed(1)
p_val=c(p_val_combine_GDP(eps=0.25, del1=1/n_m, del2=1/n_ny, del3=1/n_nj, n1=n_m, n2=n_ny, n3=n_nj, 
                          p01=H0,p02=H0,p03=H0,   x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,alpha=0.05, size=100),
        p_val_combine_GDP(eps=0.5, del1=1/n_m, del2=1/n_ny, del3=1/n_nj, n1=n_m, n2=n_ny, n3=n_nj, 
                          p01=H0,p02=H0,p03=H0,   x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,alpha=0.05, size=100),
        p_val_combine_GDP(eps=0.75, del1=1/n_m, del2=1/n_ny, del3=1/n_nj, n1=n_m, n2=n_ny, n3=n_nj, 
                          p01=H0,p02=H0,p03=H0,   x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,alpha=0.05, size=100))
round(p_val,3)#0.450 0.297 0.157


#TGDP
set.seed(5)
p_val=c(p_val_combine_GDP_trunc(eps=0.25, del1=1/n_m, del2=1/n_ny, del3=1/n_nj, n1=n_m, n2=n_ny, n3=n_nj, 
                                p01=H0,p02=H0,p03=H0,   x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,alpha=0.05, size=100),
        p_val_combine_GDP_trunc(eps=0.5, del1=1/n_m, del2=1/n_ny, del3=1/n_nj, n1=n_m, n2=n_ny, n3=n_nj, 
                                p01=H0,p02=H0,p03=H0,   x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,alpha=0.05, size=100),
        p_val_combine_GDP_trunc(eps=0.75, del1=1/n_m, del2=1/n_ny, del3=1/n_nj, n1=n_m, n2=n_ny, n3=n_nj, 
                                p01=H0,p02=H0,p03=H0,   x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,alpha=0.05, size=100))
round(p_val,3)#0.673 0.422 0.189









#######Income variable########
#list with many cells CHINCOMCP FAMQ: PK FAMILY INCOME (CATEGORICAL)
##Data cleaning
#18 categories in total.

income=real_d$CHINCOMCP
income_m=income[ind_mass]
income_ny=income[ind_NY]
income_nj=income[ind_NJersey]
income_rem=income[-c(ind_mass, ind_NY, ind_NJersey)]

income=income[is.na(income)==F] #Remove all the nan
length(income)#2753
dat_m=income_m[is.na(income_m)==F] #Remove all the nan
dat_ny=income_ny[is.na(income_ny)==F] #Remove all the nan
dat_nj=income_nj[is.na(income_nj)==F] #Remove all the nan
dat_rem=income_rem[is.na(income_rem)==F] #Remove all the nan
n_m=length(dat_m) #354
n_ny=length(dat_ny) #155
n_nj=length(dat_nj) #286
n_rem=length(dat_rem) #1958


#Change the data into a multinomial table
levels_income=unique(income)
dat_m=summary(dat_m)
dat_ny=summary(dat_ny)
dat_nj=summary(dat_nj)
dat_rem=summary(dat_rem)





#Assume H0 to be the data is simulated from multinomial(n, dat_rem/n_rem)
p0=dat_rem/n_rem


#######Intra-table merging#######
##Merge cells 1:4, 5:10, 11:18
##We first explore empirical type I errors##
#Lap
size=500
alpha=0.05
set.seed(1)
res_lap_typeI_n100=c(1,1,1)
for (i in 1:100){
  typeI_c=c(typeI_multiple_lap_intercell_merge(eps=0.25, size, n_rem, p0, alpha, merge_idx = list(1:4, 5:10, 11:18)),
            typeI_multiple_lap_intercell_merge(eps=0.5, size, n_rem, p0, alpha,  merge_idx = list(1:4, 5:10, 11:18)),
            typeI_multiple_lap_intercell_merge(eps=0.75, size, n_rem, p0, alpha,  merge_idx = list(1:4, 5:10, 11:18)))
  res_lap_typeI_n100=rbind(res_lap_typeI_n100, typeI_c)
}
res_lap_typeI_n100=res_lap_typeI_n100[2:101,]
round(apply(res_lap_typeI_n100, 2, mean)/100,3) 


#TLap
size=500
alpha=0.05
set.seed(1)
res_lap_typeI_n100_trunc=c(1,1,1)
for (i in 1:100){
  typeI_c=c(typeI_multiple_lap_intercell_merge_trunc(eps=0.25, size, n_rem, p0, alpha, merge_idx = list(1:4, 5:10, 11:18)),
            typeI_multiple_lap_intercell_merge_trunc(eps=0.5, size, n_rem, p0, alpha,  merge_idx = list(1:4, 5:10, 11:18)),
            typeI_multiple_lap_intercell_merge_trunc(eps=0.75, size, n_rem, p0, alpha,  merge_idx = list(1:4, 5:10, 11:18)))
  res_lap_typeI_n100_trunc=rbind(res_lap_typeI_n100_trunc, typeI_c)
}
res_lap_typeI_n100_trunc=res_lap_typeI_n100_trunc[2:101,]
round(apply(res_lap_typeI_n100_trunc, 2, mean)/100,3) 



#GDP
set.seed(2)
res_GDP_typeI_n100=c(1,1,1)
for (i in 1:100){
  typeI_c=c(typeI_multiple_GDP_intercell_merge_income(eps=0.25, size, n_rem, p0, alpha, merge_idx = list(1:4, 5:10, 11:18)),
            typeI_multiple_GDP_intercell_merge_income(eps=0.5, size, n_rem, p0, alpha, merge_idx = list(1:4, 5:10, 11:18)),
            typeI_multiple_GDP_intercell_merge_income(eps=0.75, size, n_rem, p0, alpha,merge_idx = list(1:4, 5:10, 11:18)))
  res_GDP_typeI_n100=rbind(res_GDP_typeI_n100, typeI_c)
}
res_GDP_typeI_n100=res_GDP_typeI_n100[2:101,]
round(apply(res_GDP_typeI_n100, 2, mean)/100,3) 


#TGDP
set.seed(1)
res_GDP_typeI_n100_trunc=c(1,1,1)
for (i in 1:100){
  typeI_c=c(typeI_multiple_GDP_intercell_merge_trunc_income(eps=0.25, size, n_rem, p0, alpha, merge_idx = list(1:4, 5:10, 11:18)),
            typeI_multiple_GDP_intercell_merge_trunc_income(eps=0.5, size, n_rem, p0, alpha, merge_idx = list(1:4, 5:10, 11:18)),
            typeI_multiple_GDP_intercell_merge_trunc_income(eps=0.75, size, n_rem, p0, alpha,merge_idx = list(1:4, 5:10, 11:18)))
  res_GDP_typeI_n100_trunc=rbind(res_GDP_typeI_n100_trunc, typeI_c)
}
res_GDP_typeI_n100_trunc=res_GDP_typeI_n100_trunc[2:101,]
round(apply(res_GDP_typeI_n100_trunc, 2, mean)/100,3) 



#Opt mechanism
size=500
alpha=0.05
pr=rep(1/(n_rem+1), n_rem+1)
opt_mat1=perturbation_matrix(n_rem, eps=0.25, pr)
opt_mat2=perturbation_matrix(n_rem, eps=0.5, pr)
opt_mat3=perturbation_matrix(n_rem, eps=0.75, pr)
set.seed(1)
res_opt_typeI_n100=c(1,1,1)
for (i in 1:100){
  typeI_c=c(typeI_opt_intercellmerge_manycell(opt_mat1,eps=0.25, size, n_rem, p0, alpha,  merge_idx = list(1:4, 5:10, 11:18)),
            typeI_opt_intercellmerge_manycell(opt_mat2,eps=0.5, size, n_rem, p0, alpha,  merge_idx = list(1:4, 5:10, 11:18)),
            typeI_opt_intercellmerge_manycell(opt_mat3,eps=0.75, size, n_rem, p0, alpha, merge_idx = list(1:4, 5:10, 11:18)))
  res_opt_typeI_n100=rbind(res_opt_typeI_n100, typeI_c)
}
res_opt_typeI_n100=res_opt_typeI_n100[2:101,]
round(apply(res_opt_typeI_n100, 2, mean)/100, 3) 


####We next explore the empirical power of the various methods####
n=length(income)
#Lap
#eps=0.25
size=500
alpha=0.05
p_v=seq(p0[4], p0[4]+0.035, 0.0025)
eps=0.25
set.seed(2)
p_n100_eps0.25_lap=c()
for (p in p_v){
  delta=p-p0[4]
  p1=c(p0[1:3], p, p0[5], p0[6]-delta, p0[7:18])
  pow=pow_multiple_lap_intercell_merge(eps, size, n, p0, alpha, p1, merge_idx = list(1:4, 5:10, 11:18))
  p_n100_eps0.25_lap=c(p_n100_eps0.25_lap, pow)
}

#eps=0.5
eps=0.5
set.seed(2)
p_n100_eps0.5_lap=c()
for (p in p_v){
  delta=p-p0[4]
  p1=c(p0[1:3], p, p0[5], p0[6]-delta, p0[7:18])
  pow=pow_multiple_lap_intercell_merge(eps, size, n, p0, alpha, p1, merge_idx = list(1:4, 5:10, 11:18))
  p_n100_eps0.5_lap=c(p_n100_eps0.5_lap, pow)
}

#eps=0.75
eps=0.75
set.seed(2)
p_n100_eps0.75_lap=c()
for (p in p_v){
  delta=p-p0[4]
  p1=c(p0[1:3], p, p0[5], p0[6]-delta, p0[7:18])
  pow=pow_multiple_lap_intercell_merge(eps, size, n, p0, alpha, p1, merge_idx = list(1:4, 5:10, 11:18))
  p_n100_eps0.75_lap=c(p_n100_eps0.75_lap, pow)
}




#TLap
#eps=0.25
size=500
alpha=0.05
eps=0.25
set.seed(2)
p_n100_eps0.25_lap_trunc=c()
for (p in p_v){
  delta=p-p0[4]
  p1=c(p0[1:3], p, p0[5], p0[6]-delta, p0[7:18])
  pow=pow_multiple_lap_intercell_merge_trunc(eps, size, n, p0, alpha, p1, merge_idx = list(1:4, 5:10, 11:18))
  p_n100_eps0.25_lap_trunc=c(p_n100_eps0.25_lap_trunc, pow)
}

#eps=0.5
eps=0.5
set.seed(2)
p_n100_eps0.5_lap_trunc=c()
for (p in p_v){
  delta=p-p0[4]
  p1=c(p0[1:3], p, p0[5], p0[6]-delta, p0[7:18])
  pow=pow_multiple_lap_intercell_merge_trunc(eps, size, n, p0, alpha, p1, merge_idx = list(1:4, 5:10, 11:18))
  p_n100_eps0.5_lap_trunc=c(p_n100_eps0.5_lap_trunc, pow)
}

#eps=0.75
eps=0.75
set.seed(2)
p_n100_eps0.75_lap_trunc=c()
for (p in p_v){
  delta=p-p0[4]
  p1=c(p0[1:3], p, p0[5], p0[6]-delta, p0[7:18])
  pow=pow_multiple_lap_intercell_merge_trunc(eps, size, n, p0, alpha, p1, merge_idx = list(1:4, 5:10, 11:18))
  p_n100_eps0.75_lap_trunc=c(p_n100_eps0.75_lap_trunc, pow)
}



#GDP
#eps=0.25
size=500
alpha=0.05
delta=1/n
eps=0.25
set.seed(1)
p_n100_eps0.25_GDP=c()
for (p in p_v){
  del=p-p0[4]
  p1=c(p0[1:3], p, p0[5], p0[6]-del, p0[7:18])
  pow=pow_multiple_GDP_intercell_merge_income(eps, size, n, p0, alpha, p1,  merge_idx = list(1:4, 5:10, 11:18))
  p_n100_eps0.25_GDP=c(p_n100_eps0.25_GDP, pow)
}

#eps=0.5
eps=0.5
delta=1/n
set.seed(1)
p_n100_eps0.5_GDP=c()
for (p in p_v){
  del=p-p0[4]
  p1=c(p0[1:3], p, p0[5], p0[6]-del, p0[7:18])
  pow=pow_multiple_GDP_intercell_merge_income(eps, size, n, p0, alpha, p1,  merge_idx = list(1:4, 5:10, 11:18))
  p_n100_eps0.5_GDP=c(p_n100_eps0.5_GDP, pow)
}

#eps=0.75
delta=1/n
set.seed(1)
eps=0.75
p_n100_eps0.75_GDP=c()
for (p in p_v){
  del=p-p0[4]
  p1=c(p0[1:3], p, p0[5], p0[6]-del, p0[7:18])
  pow=pow_multiple_GDP_intercell_merge_income(eps, size, n, p0, alpha, p1,  merge_idx = list(1:4, 5:10, 11:18))
  p_n100_eps0.75_GDP=c(p_n100_eps0.75_GDP, pow)
}




#TGDP
#eps=0.25
size=500
alpha=0.05
delta=1/n
eps=0.25
set.seed(1)
p_n100_eps0.25_GDP_trunc=c()
for (p in p_v){
  del=p-p0[4]
  p1=c(p0[1:3], p, p0[5], p0[6]-del, p0[7:18])
  pow=pow_multiple_GDP_intercell_merge_trunc_income(eps, size, n, p0, alpha, p1,  merge_idx = list(1:4, 5:10, 11:18))
  p_n100_eps0.25_GDP_trunc=c(p_n100_eps0.25_GDP_trunc, pow)
}

#eps=0.5
eps=0.5
delta=1/n
set.seed(1)
p_n100_eps0.5_GDP_trunc=c()
for (p in p_v){
  del=p-p0[4]
  p1=c(p0[1:3], p, p0[5], p0[6]-del, p0[7:18])
  pow=pow_multiple_GDP_intercell_merge_trunc_income(eps, size, n, p0, alpha, p1,  merge_idx = list(1:4, 5:10, 11:18))
  p_n100_eps0.5_GDP_trunc=c(p_n100_eps0.5_GDP_trunc, pow)
}

#eps=0.75
delta=1/n
set.seed(1)
eps=0.75
p_n100_eps0.75_GDP_trunc=c()
for (p in p_v){
  del=p-p0[4]
  p1=c(p0[1:3], p, p0[5], p0[6]-del, p0[7:18])
  pow=pow_multiple_GDP_intercell_merge_trunc_income(eps, size, n, p0, alpha, p1,  merge_idx = list(1:4, 5:10, 11:18))
  p_n100_eps0.75_GDP_trunc=c(p_n100_eps0.75_GDP_trunc, pow)
}


#Opt
#eps=0.25
size=500
alpha=0.05
pr=rep(1/(n+1), n+1)
eps=0.25
opt_mat=perturbation_matrix(n, eps, pr)
set.seed(1)
p_n100_eps0.25_opt_c=c()
for (p in p_v){
  del=p-p0[4]
  p1=c(p0[1:3], p, p0[5], p0[6]-del, p0[7:18])
  pow=pow_opt_intercellmerge_manycell(opt_mat, eps, size, n, p0, alpha, p1,  merge_idx = list(1:4, 5:10, 11:18))
  p_n100_eps0.25_opt_c=c(p_n100_eps0.25_opt_c, pow)
}

#eps=0.5
eps=0.5
p_n100_eps0.5_opt_c=c()
opt_mat=perturbation_matrix(n, eps, pr)
set.seed(1)
for (p in p_v){
  del=p-p0[4]
  p1=c(p0[1:3], p, p0[5], p0[6]-del, p0[7:18])
  pow=pow_opt_intercellmerge_manycell(opt_mat,eps, size, n, p0, alpha,  p1,merge_idx = list(1:4, 5:10, 11:18))
  p_n100_eps0.5_opt_c=c(p_n100_eps0.5_opt_c, pow)
}

#eps=0.75
eps=0.75
opt_mat=perturbation_matrix(n, eps, pr)
set.seed(1)
p_n100_eps0.75_opt_c=c()
for (p in p_v){
  del=p-p0[4]
  p1=c(p0[1:3], p, p0[5], p0[6]-del, p0[7:18])
  pow=pow_opt_intercellmerge_manycell(opt_mat,eps, size, n, p0, alpha,  p1,merge_idx = list(1:4, 5:10, 11:18))
  p_n100_eps0.75_opt_c=c(p_n100_eps0.75_opt_c, pow)
}



pdf("real_power_income_intercell_merge_latest_new.pdf", width=12, height=4)
#par(mfrow=c(1,3),cex=0.7, mai=c(0.6,0.6,0.6,0.6))
par(mfrow=c(1,3),cex=0.7, mai=c(0.6,0.25,0.2,0.2))
plot(p_v, p_n100_eps0.25_opt_c/100, type='o', ylim=c(0,1),
     xlab='Alternative Hypothesis (p4)', ylab='Empirical power', pch='o',
     main = 'Epsilon=0.25', lty=1, col='blue')
points(p_v,  p_n100_eps0.25_lap/100, col="red", pch="*")
lines(p_v, p_n100_eps0.25_lap/100, col="red",lty=2)
points(p_v,  p_n100_eps0.25_lap_trunc/100, col="darkgoldenrod1",pch=16)
lines(p_v,  p_n100_eps0.25_lap_trunc/100, col="darkgoldenrod1", lty=5)
points(p_v,  p_n100_eps0.25_GDP/100, col="black",pch="+")
lines(p_v,  p_n100_eps0.25_GDP/100, col="black", lty=3)
points(p_v,  p_n100_eps0.25_GDP_trunc/100, col="purple",pch=5)
lines(p_v,  p_n100_eps0.25_GDP_trunc/100, col="purple", lty=4)

legend(0.12,1,legend=c("Opt","Lap","TLap","GDP","TGDP"), col=c("blue","red","darkgoldenrod1","black","purple"),
       pch=c("o","*",16, "+",5),lty=c(1,2,5, 3,4), ncol=1,cex=0.75)

plot(p_v, p_n100_eps0.5_opt_c/100, type='o', ylim=c(0,1),
     xlab='Alternative Hypothesis (p4)', ylab='Empirical power', pch='o',
     main = 'Epsilon=0.5', lty=1, col='blue')
points(p_v,  p_n100_eps0.5_lap/100, col="red", pch="*")
lines(p_v, p_n100_eps0.5_lap/100, col="red",lty=2)
points(p_v,  p_n100_eps0.5_lap_trunc/100, col="darkgoldenrod1",pch=16)
lines(p_v,  p_n100_eps0.5_lap_trunc/100, col="darkgoldenrod1", lty=5)
points(p_v,  p_n100_eps0.5_GDP/100, col="black",pch="+")
lines(p_v,  p_n100_eps0.5_GDP/100, col="black", lty=3)
points(p_v,  p_n100_eps0.5_GDP_trunc/100, col="purple",pch=5)
lines(p_v,  p_n100_eps0.5_GDP_trunc/100, col="purple", lty=4)

legend(0.12,1,legend=c("Opt","Lap","TLap","GDP","TGDP"), col=c("blue","red","darkgoldenrod1","black","purple"),
       pch=c("o","*",16, "+",5),lty=c(1,2,5, 3,4), ncol=1,cex=0.75)

plot(p_v, p_n100_eps0.75_opt_c/100, type='o', ylim=c(0,1),
     xlab='Alternative Hypothesis (p4)', ylab='Empirical power', pch='o',
     main = 'Epsilon=0.75', lty=1, col='blue')
points(p_v,  p_n100_eps0.75_lap/100, col="red", pch="*")
lines(p_v, p_n100_eps0.75_lap/100, col="red",lty=2)
points(p_v,  p_n100_eps0.75_lap_trunc/100, col="darkgoldenrod1",pch=16)
lines(p_v,  p_n100_eps0.75_lap_trunc/100, col="darkgoldenrod1", lty=5)
points(p_v,  p_n100_eps0.75_GDP/100, col="black",pch="+")
lines(p_v,  p_n100_eps0.75_GDP/100, col="black", lty=3)
points(p_v,  p_n100_eps0.75_GDP_trunc/100, col="purple",pch=5)
lines(p_v,  p_n100_eps0.75_GDP_trunc/100, col="purple", lty=4)

legend(0.12,1,legend=c("Opt","Lap","TLap","GDP","TGDP"), col=c("blue","red","darkgoldenrod1","black","purple"),
       pch=c("o","*",16, "+",5),lty=c(1,2,5, 3,4), ncol=1,cex=0.75)
dev.off()





###We compare p-values using the observed value and the private data simulated from various methods###
####Using observed value####
#1:4, 5:10, 11:18
dat_m_merge=c(sum(dat_m[1:4]), sum(dat_m[5:10]), sum(dat_m[11:18]))
dat_ny_merge=c(sum(dat_ny[1:4]), sum(dat_ny[5:10]), sum(dat_ny[11:18]))
dat_nj_merge=c(sum(dat_nj[1:4]), sum(dat_nj[5:10]), sum(dat_nj[11:18]))
p0_merge=c(sum(p0[1:4]), sum(p0[5:10]), sum(p0[11:18]))
set.seed(1)
t_true=sum((dat_m_merge-n_m*p0_merge)^2/(n_m*p0_merge))+sum((dat_ny_merge-n_ny*p0_merge)^2/(n_ny*p0_merge))+sum((dat_nj_merge-n_nj*p0_merge)^2/(n_nj*p0_merge))
sts=rchisq(100000, 2)+rchisq(100000, 2)+rchisq(100000, 2)
sts=sort(sts)
1-which(sts>t_true)[1]/100000#NA, i.e. 0

#Lap
set.seed(1)
p_value=c(p_value_multiple_lap_intercell_merge(eps=0.25, size=500, n1=n_m, n2=n_ny, n3=n_nj, p0=dat_rem/n_rem, 
                                               alpha=0.05, x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,merge_idx=list(1:4,5:10,11:18)),
          p_value_multiple_lap_intercell_merge(eps=0.5, size=500, n1=n_m, n2=n_ny, n3=n_nj, p0=dat_rem/n_rem, 
                                               alpha=0.05, x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,merge_idx=list(1:4,5:10,11:18)),
          p_value_multiple_lap_intercell_merge(eps=0.75, size=500, n1=n_m, n2=n_ny, n3=n_nj, p0=dat_rem/n_rem, 
                                               alpha=0.05, x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,merge_idx=list(1:4,5:10,11:18)))

round(p_value,3)         


#TLap
set.seed(1)
p_value=c(p_value_multiple_lap_intercell_merge_trunc(eps=0.25, size=500, n1=n_m, n2=n_ny, n3=n_nj, p0=dat_rem/n_rem, 
                                                     alpha=0.05, x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,merge_idx=list(1:4,5:10,11:18)),
          p_value_multiple_lap_intercell_merge_trunc(eps=0.5, size=500, n1=n_m, n2=n_ny, n3=n_nj, p0=dat_rem/n_rem, 
                                                     alpha=0.05, x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,merge_idx=list(1:4,5:10,11:18)),
          p_value_multiple_lap_intercell_merge_trunc(eps=0.75, size=500, n1=n_m, n2=n_ny, n3=n_nj, p0=dat_rem/n_rem, 
                                                     alpha=0.05, x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,merge_idx=list(1:4,5:10,11:18)))

round(p_value,3)         



#GDP
set.seed(1)
p_value=c(p_value_multiple_GDP_intercell_merge_income(eps=0.25, size=500, n1=n_m, n2=n_ny, n3=n_nj, p0=dat_rem/n_rem, 
                                                      alpha=0.05, x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,merge_idx=list(1:4,5:10,11:18)),
          p_value_multiple_GDP_intercell_merge_income(eps=0.5, size=500, n1=n_m, n2=n_ny, n3=n_nj, p0=dat_rem/n_rem, 
                                                      alpha=0.05, x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,merge_idx=list(1:4,5:10,11:18)),
          p_value_multiple_GDP_intercell_merge_income(eps=0.75, size=500, n1=n_m, n2=n_ny, n3=n_nj, p0=dat_rem/n_rem, 
                                                      alpha=0.05, x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,merge_idx=list(1:4,5:10,11:18)))

round(p_value ,3)        
#0.038 0.000 0.000

#Truncated GDP
set.seed(1)
p_value=c(p_value_multiple_GDP_intercell_merge_trunc_income(eps=0.25, size=500, n1=n_m, n2=n_ny, n3=n_nj, p0=dat_rem/n_rem, 
                                                            alpha=0.05, x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,merge_idx=list(1:4,5:10,11:18)),
          p_value_multiple_GDP_intercell_merge_trunc_income(eps=0.5, size=500, n1=n_m, n2=n_ny, n3=n_nj, p0=dat_rem/n_rem, 
                                                            alpha=0.05, x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,merge_idx=list(1:4,5:10,11:18)),
          p_value_multiple_GDP_intercell_merge_trunc_income(eps=0.75, size=500, n1=n_m, n2=n_ny, n3=n_nj, p0=dat_rem/n_rem, 
                                                            alpha=0.05, x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,merge_idx=list(1:4,5:10,11:18)))

round(p_value ,3) 


#Opt
p1=rep(1/(n_m+1), n_m+1)
p2=rep(1/(n_ny+1), n_ny+1)
p3=rep(1/(n_nj+1), n_nj+1)
p_value=c(p_value_opt_intercellmerge_manycell_income(eps=0.25, size=500, n1=n_m, n2=n_ny, n3=n_nj, p0=dat_rem/n_rem, 
                                                     alpha=0.05, p1, p2, p3, x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,merge_idx=list(1:4,5:10,11:18)),
          p_value_opt_intercellmerge_manycell_income(eps=0.5, size=500, n1=n_m, n2=n_ny, n3=n_nj, p0=dat_rem/n_rem, 
                                                     alpha=0.05, p1, p2, p3, x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,merge_idx=list(1:4,5:10,11:18)),
          p_value_opt_intercellmerge_manycell_income(eps=0.75, size=500, n1=n_m, n2=n_ny, n3=n_nj, p0=dat_rem/n_rem, 
                                                     alpha=0.05, p1, p2, p3, x_value1=dat_m, x_value2=dat_ny, x_value3=dat_nj,merge_idx=list(1:4,5:10,11:18)))

round(p_value,3)      




























